@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##############################################################
 # smscon_master - remote control master for Nokia N900 phone #
-# $Id: smscon_master.py 196 2012-06-08 18:44:59Z yablacky $
+# $Id: smscon_master.py 221 2012-07-01 21:57:32Z yablacky $
 ##############################################################
 
 import os
@@ -62,33 +62,75 @@ def ShowSmsCommands(theConf):
         print theConf._genConfigLine(SmsCommand, theConf['SMS_COMPREFIX'] + theConf[SmsCommand] + theConf['SMS_COMSUFFIX'])
     return True
 
-def ShowIMSICodes():
+
+def AuthorizedSIMs_ShowList():
     """
-    Show IMSI numbers from IMSI database.
-    @return bool True iff IMSI codes available and listed.
+    Show list of authorized SIM cards from SIM database.
+    @return bool True iff SIM cards available and listed.
     """
-    imsi = IMSIConfiguration()
-    if not imsi.load():
+    simDB = IMSIConfiguration()
+    if not simDB.load():
         return False
-    for n, Text in enumerate(imsi):
-        print 'IMSI_%02d = %s' % ( n + 1, Text )
+    for n, Text in enumerate(simDB):
+        CodeType, CodeValue = simDB.UnpackCodeType(Text)
+        print 'SIM_%02d = %s %s' % ( n + 1, CodeType, CodeValue )
     return True;
 
-def SaveIMSIcode():
+def AuthorizedSIMs_GrantCurrentSIM():
     """
-    Get current inserted SIM IMSI code & save to IMSI database.
-    @return bool True iff IMSI code written to IMSI database.
+    Authorize the currently inserted SIM card to SIM database and
+    tell daemon to reload SIM database.
+    @return bool True iff SIM card data written to SIM database.
     """
-    imsi = IMSIConfiguration()
-    return imsi.update(GetPhoneCode('IMSI'), '')
+    ICCID = GetPhoneCode('ICCID')
+    if ICCID == None:
+        LOGGER.error('No SIM card present.')
+        return False
+    IMSI = GetPhoneCode('IMSI')
+    if IMSI == None:
+        LOGGER.error('The SIM card is not active (no or wrong PIN).')
+        return False
+    simDB = IMSIConfiguration()
+    simDB.updateCode('IMSI', IMSI, None, Silent = True) # Ensure IMSI code of this SIM is removed from database.
+    ok = simDB.updateCode('ICCID', ICCID, '')
+    if ok:
+        daemon.tell(daemon.TELL_RELOAD)
+    return ok
 
-def RemoveIMSIcode():
+def AuthorizedSIMs_RevokeCurrentSIM():
     """
-    Remove current inserted SIM IMSI code from IMSI database.
-    @return bool True iff IMSI code no longer present in IMSI database.
+    Remove current inserted SIM card code from SIM database and
+    tell daemon to reload SIM database.
+    @return bool True iff current SIM code no longer present in SIM database.
     """
-    imsi = IMSIConfiguration()
-    return imsi.update(GetPhoneCode('IMSI'), None)
+    ICCID = GetPhoneCode('ICCID')
+    if ICCID == None:
+        LOGGER.error('No SIM card present.')
+        return False
+    simDB = IMSIConfiguration()
+    ok1 = simDB.updateCode('ICCID', ICCID, None)
+    IMSI = GetPhoneCode('IMSI')
+    if IMSI == None:
+        ok2 = False
+        LOGGER.warning('The SIM card is not active (no or wrong PIN); could still be authorized!')
+    else:
+        ok2 = simDB.updateCode('IMSI', IMSI, None, Silent = True)
+    if ok1 or ok2:
+        daemon.tell(daemon.TELL_RELOAD)
+    return ok
+
+def AuthorizedSIMs_RevokeAllSIM():
+    """
+    Remove all SIM card codes from SIM database and
+    tell daemon to reload SIM database. 
+    @return bool True iff no SIM codes present in SIM database.
+    """
+    if not IsFileReadable(InstPath, CodeFile):
+        return True
+    ok = DeleteFile(InstPath, CodeFile)
+    if ok:
+        daemon.tell(daemon.TELL_RELOAD)
+    return ok
 
 ##############################################################
 # Command line interface
@@ -101,22 +143,27 @@ def ShowOptions(Level = 1):
         print '== %s %s - Nokia N900 remote control utility ==' % (NAME, VERSION)
         print 'usage: %s [Options]...' % NAME
         print ' Options:'
-        print '   -start             : Start %s' % DaemonFile
-        print '   -restart           : Stop & restart %s' % DaemonFile
-        print '   -stop              : Stop %s' % DaemonFile
-        print '   -status            : Get %s status' % DaemonFile
+        print '   -start             : Start %s.' % DaemonFile
+        print '   -restart           : Stop & restart %s.' % DaemonFile
+        print '   -stop              : Stop %s.' % DaemonFile
+        print '   -status            : Get %s status.' % DaemonFile
         print '   -log               : Show log file. Anonymized if no or wrong -pw option seen before.'
-        print '   -del log           : Delete log file'
-        print '   -sms               : Show SMS commands'
-        print '   -set name "value"  : Set user setting (name = "value")'
-        print '   -config            : Show %s file' % ConfigDisplayName
-        print '   -script            : Show %s file' % ScriptDisplayName
-        print '   -scriptlog         : Show %s file' % ScriptLogDisplayName
-        print '   -del scriptlog     : Delete %s file' % ScriptLogDisplayName
-        print '   -boot              : Enable %s to start at phone boot' % DaemonFile
-        print '   -unboot            : Disable %s to start at phone boot' % DaemonFile
-        print '   -add imsi          : Add IMSI code of current SIM to %s file' % CodeDisplayName
-        print '   -remove imsi       : Remove IMSI code of current SIM from %s file' % CodeDisplayName
+        print '   -del log           : Delete log file.'
+        print '   -sms               : Show SMS commands.'
+        print '   -set name "value"  : Set user setting (name = "value").'
+        print '   -alarm [ref] "filename" : Define sound to be played on alarm command.'
+        print '                        If ref is specified the file will only referenced, not copied.'
+        print '                        Use ref only while testing several sounds.' 
+        print '                        A filename of --default restores standard alarm.'
+        print '   -config            : Show %s file.' % ConfigDisplayName
+        print '   -script            : Show %s file.' % ScriptDisplayName
+        print '   -scriptlog         : Show %s file.' % ScriptLogDisplayName
+        print '   -del scriptlog     : Delete %s file.' % ScriptLogDisplayName
+        print '   -boot              : Enable %s to start at phone boot.' % DaemonFile
+        print '   -unboot            : Disable %s to start at phone boot.' % DaemonFile
+        print '   -add imsi          : Grant authorization to current SIM card.'
+        print '   -remove imsi       : Revoke authorization of current SIM card.'
+        print '   -del imsi          : Revoke authorization of all SIM cards.'
         print '   -pw password       : Password to enable non-anonymized output. MASTERNUMBER is password.'
         print '   -version           : Show version number'    
         print '   -help              : This help menu'
@@ -133,10 +180,10 @@ def ShowOptions(Level = 1):
         print '                        Use -export! to write even existing file.'
         print '   -import filename   : Import user settings from filename.'
         print '                        Use -import! to continue import on errors.'
-        print '   -imsi              : Show IMSI code file'
-        print '   -del imsi          : Delete %s file' % CodeDisplayName
-        print '   -del config        : Delete %s file' % ConfigDisplayName
-        print '   -del script        : Delete %s file' % ScriptDisplayName
+        print '   -imsi              : Show list of authorized SIM cards.'
+        print '   -del imsi          : Revoke authorization from all SIM cards.'
+        print '   -del config        : Delete %s file.' % ConfigDisplayName
+        print '   -del script        : Delete %s file.' % ScriptDisplayName
         print '   -sendsms number message : Send one SMS message to number. Number may be verbatim "MASTERNUMBER"'
         print '   -sendemail recipient subject message [attachment-filename [attachment-mimetype]]: Send email.'
         print '                        If recipient is a single char the EMAILADDRESS from configuration is used.'
@@ -150,15 +197,15 @@ def ShowHiddenOptions():
     Show hidden options for smscon.
     """
     print ' Developer options:'
-    print '   -devhelp           : This extended help menu'
+    print '   -devhelp           : This extended help menu.'
     print '   -branch            : Show branch number. Deprecated. Removed in future. Shows -version now.'
     print '   -build             : Show build number.  Deprecated. Removed in future. Shows -version now.'
     print '   -init!             : Create default %s files. Do not upgrade existing ones.' % ' & '.join((ConfigDisplayName, ScriptDisplayName))
     print '   -reload            : Tell daemon to read config file again (-force-reload is also accepted).'
-    print '   -get name "default": Get user setting'
-    print '   -test "test"       : Run developer test (gps1, gps2, ssh, ssh-off, cam, sms, email1, email2, call, script)'
-    print '   -comtest "command" : Run %s command as if received as SMS message (no prefix/suffix required)' % NAME
-    print '   -devconfig password: Show uncrypted %s file. MASTERNUMBER is password' % ConfigDisplayName
+    print '   -get name "default": Get user setting.'
+    print '   -test "test"       : Run developer test (gps1, gps2, ssh, ssh-off, cam, sms, email1, email2, call, script).'
+    print '   -comtest "command" : Run %s command as if received as SMS message (no prefix/suffix required).' % NAME
+    print '   -devconfig password: Show uncrypted %s file. MASTERNUMBER is password.' % ConfigDisplayName
 
 ##############################################################
 
@@ -209,7 +256,7 @@ def DoCommandLine():
                     ('([a-zA-Z][a-zA-Z0-9_\\-]*\\.)+[a-zA-Z0-9_\\-]*[a-zA-Z]', 'example.com'),  # replace potential host names
                     ('(?:[0-9]{1,3}\\.){2,3}([0-9]{1,3})', '192.168.128.\\1'),  # replace potential IPv4 (but no floats)
                 ])    
-        elif Mode == '-imsi':       ok = ShowIMSICodes()
+        elif Mode == '-imsi':       ok = AuthorizedSIMs_ShowList()
         elif Mode == '-sms':        ok = ShowSmsCommands(ourConf)
         elif Mode == '-config':     ok = ShowFile(ourConf.Filename)
         elif Mode == '-script':     ok = ShowFile(script.Filename)
@@ -313,7 +360,7 @@ def DoCommandLine():
             ok = DeleteFile(InstPath, ScriptLog) and ok
             ok = DeleteFile(InstPath, PhotoName) and ok
             ok = DeleteFile(InstPath, MapName) and ok
-            ok = DeleteFile(InstPath, AudioFile) and ok
+            ok = DeleteFile(InstPath, AudioFile) and ok     # don't delete DefaultAudioFile.
             ok = DeleteFile(LogPath, LogFile) and ok
             ok = DeleteFile(LogPath, BootLogFile) and ok
 
@@ -329,11 +376,49 @@ def DoCommandLine():
                 return TellError('%s %s failed to start.' % (Mode[1:], Func))
             LOGGER.info('%s %s started...' % (Mode[1:], Func))
 
+        elif Mode == '-alarm':
+            if len(args) == 0: return TellError("%s requires argument." % Mode)
+            Ref = args.pop(0)
+            if Ref == 'ref':
+                if len(args) == 0: return TellError("%s ref requires argument." % Mode)
+                NewAudioFilename = args.pop(0)
+            else:
+                NewAudioFilename, Ref = Ref, False 
+
+            if NewAudioFilename == '--default':
+                NewAudioFilename, Ref = os.path.join(InstPath, DefaultAudioFile), True
+            if Ref:
+                NewAudioFilename = os.path.abspath(NewAudioFilename)
+            if not IsFileReadable(NewAudioFilename):
+                return TellError("Sound does not exist or is not readable: %s" % NewAudioFilename)
+            # Remove the old audio file, do not just write! It could be a symlink.
+            Problem = ''
+            Filename = os.path.join(InstPath, AudioFile)
+            try:    os.remove(Filename)
+            except Exception, e:
+                Problem = str(e)
+            if IsFileReadable(Filename):
+                return TellError("Failed to remove old alarm sound '%s'. %s" % (Filename, Problem))
+            # Establish the new file:
+            try:
+                if Ref:
+                    os.symlink(NewAudioFilename, Filename)
+                else:
+                    os.system('cp %s %s' % (shellArg(NewAudioFilename), shellArg(Filename)))
+            except Exception, e:
+                Problem = str(e)
+            if not IsFileReadable(Filename):
+                return TellError("Failed to establish alarm sound '%s'. %s" % (NewAudioFilename, Problem))
+            LOGGER.info("Alarm sound '%s' established." % (NewAudioFilename))
+
         elif Mode == '-del':
             if len(args) == 0: return TellError("%s requires argument." % Mode)
             Func = args.pop(0)
-            if   Func == 'log':         ok = DeleteFile(LogPath, LogFile)
-            elif Func == 'imsi':        ok = DeleteFile(InstPath, CodeFile)
+            if   Func == 'log':
+                                for fname in os.listdir(LogPath):
+                                    if fname.startswith(LogFile):    # Account for files generated by RotatingFileHandler.
+                                        ok = DeleteFile(LogPath, fname) and ok
+            elif Func == 'imsi':        ok = AuthorizedSIMs_RevokeAllSIM()
             elif Func == 'config':      ok = DeleteFile(InstPath, ConfigFile)
             elif Func == 'script':      ok = DeleteFile(script.Filename)
             elif Func == 'scriptlog':   ok = DeleteFile(InstPath, ScriptLog)
@@ -342,13 +427,13 @@ def DoCommandLine():
         elif Mode == '-add':
             if len(args) == 0: return TellError("%s requires argument." % Mode)
             Func = args.pop(0)
-            if   Func == 'imsi':    ok = SaveIMSIcode()
+            if   Func == 'imsi':    ok = AuthorizedSIMs_GrantCurrentSIM()
             else:   return TellError('%s: unknown option (%s).' % (Mode, Func))
 
-        elif Mode == '-remove': # add current IMSI code to smscon_code file
+        elif Mode == '-remove':
             if len(args) == 0: return TellError("%s requires argument." % Mode)
             Func = args.pop(0)
-            if   Func == 'imsi':    ok = RemoveIMSIcode()
+            if   Func == 'imsi':    ok = AuthorizedSIMs_RevokeCurrentSIM()
             else:   return TellError('%s: unknown option (%s).' % (Mode, Func))
 
         elif Mode == '-set':
