@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 ##############################################################
 # smscon - remote control library for Nokia N900 phone       #
-# $Id: smsconlib.py 198 2012-06-08 18:52:49Z yablacky $
+# $Id: smsconlib.py 221 2012-07-01 21:57:32Z yablacky $
 ##############################################################
 
 # standard modules
@@ -9,6 +9,7 @@ import os
 import re
 #import dbus
 import logging
+import logging.handlers # for RotatingFileHandler etc.
 import fileinput
 import signal
 import time
@@ -34,7 +35,7 @@ NAME = 'smscon'
 VERSION = '999.1'   # for trunk and private builds etc.
 
 NAME, VERSION = getSysBuildNameAndVersion(NAME, VERSION,
-    headerInfo  = '$Header: https://vcs.maemo.org/svn/smscon/branches/smscon_0.10.3/src/opt/smscon/smsconlib.py 198 2012-06-08 18:52:49Z yablacky $',
+    headerInfo  = '$Header: https://vcs.maemo.org/svn/smscon/branches/smscon_0.10.6/src/opt/smscon/smsconlib.py 221 2012-07-01 21:57:32Z yablacky $',
     dirLevels   = 3 )
     
 XDEBUG   = False        # cross debug (use only if not running on phone)
@@ -55,7 +56,8 @@ BootFile    = '%s' % NAME       # This will be the system wide job name.
 
 PhotoName   = 'frontcam.jpg'
 MapName     = 'gpsmap.png'
-AudioFile   = 'alarm.wav'
+DefaultAudioFile   = 'alarm.wav'
+AudioFile   = '%s_alarm.sound' % NAME
 
 LogPath    = InstPath
 LogFile    = '%s.log' % NAME
@@ -290,13 +292,15 @@ class Configuration(dict):
     Base class for file based configuration databases
     """
     ItemTypeName        = "configuration"
+    Thing               = None      # --> use default from ItemTypeName and Filename
+    Silent              = False
     CreateTemplate      = ''
     ConfigPrintFormat   = "%s = %s"                                     # (ConfigVar, QuotedConfigValue)
     ConfigParseFormat   = "^\\s*(\\S+)\\s*=\\s*([\"\']|)(.*?)\\2\\s*$"  # (ConfigVar, QuoteUsed, ConfigValue)
     CommentLineFormat   = "^\\s*(#.*)?$"
     CommentMakeFormat   = '# %s'
     DefaultValueFormat  = None
-    TabSize             = 0     # 0 means: calc dynamically on _ConfigVars
+    TabSize             = 0     # 0 means: calculate dynamically on _ConfigVars
     _OPT_CRYPT          = 'E'
     _OPT_PLAIN          = 'P'
     _ConfigVars         = {}    # known items; no restriction if empty. Structure: key: (re_pattern_of_value, default_value, options, comment).
@@ -310,6 +314,9 @@ class Configuration(dict):
             if  self.TabSize == 0:
                 self.TabSize = 18
 
+    def _thing(self, Thing = None):
+        return Thing or self.Thing or '%s file "%s"' % (self.ItemTypeName, self.Filename)
+
     def copy(self):
         """@return a shallow copy (not a deep copy)"""
         other = self.__class__()
@@ -319,16 +326,30 @@ class Configuration(dict):
             setattr(other, attr, value)
         return other 
 
+    def _setSilent(self, Silent):
+        self.Silent = Silent
+        if self.Silent:
+            self.SilentLog = []
+
     def _error(self, Message, ReturnCode = False):
-        LOGGER.error(Message)
+        if self.Silent:
+            self.SilentLog.append(('E', Message))
+        else:
+            LOGGER.error(Message)
         return ReturnCode
 
     def _warning(self, Message, ReturnCode = False):
-        LOGGER.warning(Message)
+        if self.Silent:
+            self.SilentLog.append(('W', Message))
+        else:
+            LOGGER.warning(Message)
         return ReturnCode
 
     def _info(self, Message, ReturnCode = False):
-        LOGGER.info(Message)
+        if self.Silent:
+            self.SilentLog.append(('I', Message))
+        else:
+            LOGGER.info(Message)
         return ReturnCode
 
     def _getPatternMismatchHint(self, ConfigVar, Pattern):
@@ -356,19 +377,24 @@ class Configuration(dict):
         Load the configuration from file; optionally Silent about errors; optionally PrintContent while loading.
         @return bool Success indicator. The previous content of the configuration is lost in any case.
         """
+        self._setSilent(Silent)
+        ok = self._load(PrintContent = PrintContent)
+        self._setSilent(False)
+        return ok
+
+    def _load(self, PrintContent = False):
         self.clear()
         self._upgradableConfigVars = {}
-        Thing = '%s file "%s"' % (self.ItemTypeName, self.Filename)
+        Thing = self._thing()
         try:
             f = open(self.Filename, 'r')
         except:
-            if not Silent: self._error('The %s could not be found.' % Thing)
-            return False
+            return self._error('The %s could not be found.' % Thing)
+
         ConfigLines = [line.replace("\n","").replace("\r","") for line in f.readlines()] 
         f.close()
         if ConfigLines == []:
-            if not Silent: self._info('The %s is empty.' % Thing)
-            return False
+            return self._info('The %s is empty.' % Thing, False)
 
         ConfigValuePattern = re.compile(self.ConfigParseFormat)
         CommentLinePattern = re.compile(self.CommentLineFormat)
@@ -429,14 +455,19 @@ class Configuration(dict):
         @param string|None Thing Non-default description of what is being exported.
         @return bool Success indicator
         """
-        if not Thing:
-            Thing = '%s file "%s"' % (self.ItemTypeName, Filename)
+        self._setSilent(Silent)
+        ok = self._backup(Filename, Thing)
+        self._setSilent(False)
+        return ok
+
+    def _backup(self, Filename, Thing = None):        
+        Thing = self._thing(Thing)
         ok = True
         try:
             f = open(Filename, 'w')
         except Exception, e:
-            if not Silent: self._error('Export failed to open %s for writing.' % Thing)
-            return False
+            return self._error('Export failed to open %s for writing.' % Thing)
+
         try:
             temp = self.copy()
             keysToBackup = sorted(self._ConfigVars)
@@ -449,7 +480,7 @@ class Configuration(dict):
                 if ConfigVar in self:
                     ConfigValue = self[ConfigVar]
                 else:
-                    if not Silent:
+                    if not self.Silent:    # don't change ok if silent
                         ok = self._warning('Value of %s "%s" not defined. Using default.' % (self.ItemTypeName, ConfigVar))
                     try:
                         ConfigValue = temp._calcDefaultValue(DefaultValue)
@@ -461,7 +492,7 @@ class Configuration(dict):
                         ConfigValue = 'default value calculation error'
                 
                 if Pattern and not re.match(Pattern, ConfigValue):
-                    if not Silent:
+                    if not self.Silent:    # don't change ok if silent
                         ok = self._warning('Value of %s "%s" is invalid. ' % (self.ItemTypeName, ConfigVar)
                                                         + self._getPatternMismatchHint(ConfigVar, Pattern))
     
@@ -470,12 +501,10 @@ class Configuration(dict):
                 temp[ConfigVar] = ConfigValue
 
         except Exception, e:
-            if not Silent:
-                self._error('Export to %s failed: %s' % (Thing, e))
-            ok = False
+            ok = self._error('Export to %s failed: %s' % (Thing, e))
         finally:
             f.close
-        if ok and not Silent:
+        if ok:
             self._info('Export to %s successful.' % Thing)
         return ok
 
@@ -489,25 +518,27 @@ class Configuration(dict):
         @param string|None Thing Non-default description of what is being imported.
         @return int Number of imported settings (even if they are not really set to a new value). Negative on errors.
         """
-        if not Thing:
-            Thing = '%s file "%s"' % (self.ItemTypeName, Filename)
+        self._setSilent(Silent)
+        ok = self._restore(Filename, Force = Force, Thing = Thing)
+        self._setSilent(False)
+        return ok
+
+    def _restore(self, Filename, Force = False, Thing = None):        
+        Thing = self._thing(Thing)
         importCount = 0
         source = self.__class__(Filename = Filename)
-        if not source.load(Silent = Silent):
-            if not Silent:
-                self._error('Importing %s failed.' % Thing)
+        if not source.load(Silent = self.Silent):
+            self._error('Importing %s failed.' % Thing)
             return importCount
         ok = True
         for ConfigVar in sorted(source.keys()):
             if self.update(ConfigVar, source[ConfigVar]):
                 importCount += 1
                 continue
-            ok = False
-            if not Silent:
-                self._error('Importing %s failed for "%s".' % (Thing, ConfigVar))
+            ok = self._error('Importing %s failed for "%s".' % (Thing, ConfigVar))
             if not Force:
                 break
-        if ok and not Silent:
+        if ok:
             self._info('Importing %s successful.' % Thing)
         return IF(ok, importCount, -importCount)
 
@@ -539,11 +570,17 @@ class Configuration(dict):
         """@return bool Indicator if a given value is the initial value of given config var.""" 
         return IF(ConfigVar in self._ConfigVars, self._ConfigVars[ConfigVar][2] == ConfigValue, None)
 
-    def update(self, ConfigVar, ConfigValue):
+    def update(self, ConfigVar, ConfigValue, Silent = False):
         """
         Set a config var to a new value. If required add or (if ConfigValue is None) remove the variable from configuration.
         @return bool True iff the setting was known and has been updated (or added or removed).
         """
+        self._setSilent(Silent)
+        ok = self._update(ConfigVar, ConfigValue)
+        self._setSilent(False)
+        return ok
+
+    def _update(self, ConfigVar, ConfigValue):        
         if ConfigVar == None:
             return self._error('Missing %s.' % (self.ItemTypeName))
 
@@ -599,7 +636,7 @@ class Configuration(dict):
                                 continue;   # remove double entry
                             # Here it is: replace Line by UpdatedLine and consume UpdatedLine
                             Line, UpdatedLine = UpdatedLine, None
-                        elif FoundVar in self._ConfigVars and not FoundVar in self:
+                        elif FoundVar in self._ConfigVars and not (FoundVar in self):
                             # This line defines a different known config variable which
                             # is not yet load from the file. Load it implicitely if that
                             # can be done without errors (mainly for @APPVERSION key):            
@@ -686,7 +723,7 @@ class Configuration(dict):
                 del(self[key])
         return loadCount
 
-    def create(self, Template = None, Force = False):
+    def create(self, Template = None, Force = False, Load = True):
         """
         Create a new configuration file that contains all known config vars with their default value.
         Optionally use a given Template; optionally Force overwrite existing config files.  
@@ -694,13 +731,13 @@ class Configuration(dict):
         """
         if  Template == None:
             Template = self.CreateTemplate; 
-        Thing = '%s file "%s"' % (self.ItemTypeName, self.Filename)
+        Thing = self._thing()
         if not IsFileReadable(self.Filename):
             self._info('Creating new %s.' % Thing)
         elif Force:
             self._warning('Forced to create new %s.' % Thing)
         else:
-            self._info('The %s already exists. Not initialized, left unchanged.' % Thing)
+            self._info('The %s already exists. Not changed.' % Thing)
             return False
 
         try:
@@ -709,7 +746,7 @@ class Configuration(dict):
             return self._error('Failed to create %s.' % Thing)
         f.writelines(Template)
         f.close()
-        return self.load() and self.upgrade() > 0
+        return (not Load) or (self.load() and self.upgrade() > 0)
 
 ##############################################################
 
@@ -773,6 +810,9 @@ class UserConfiguration(Configuration):
         'COM_ALARM':        (_CmdPattern,       'Alarm',            _OPT_PLAIN, '# Command to play predefined alarm.wav:'),
         'COM_RESTART':      (_CmdPattern,       'Restart',          _OPT_PLAIN, '# Command to restart the daemon:'),
         'COM_LOG':          (_CmdPattern,       'Log',              _OPT_CRYPT, '# Command to ask for application log (encrypted):'),
+        'COM_SILENTON':     (_CmdPattern,       'Silent',           _OPT_PLAIN, '# Command to make phone silent on incomming commands:'),
+        'COM_SILENTOFF':    (_CmdPattern,       'NotSilent',        _OPT_PLAIN, '# Command to undo silence and restore phone audibility:'),
+        'COM_STOLEN':       (_CmdPattern,       'Stolen',           _OPT_PLAIN, '# Command to permanently enable stolen mode:'),
 
         # Prefix/suffix for all command strings (in SMS message only): 
         'SMS_COMPREFIX':    (_CmdPattern,       '',                 _OPT_CRYPT, '# Prefix that must be present before all COM_XXX commands [even COM_SHELL] (encrypted):'),
@@ -820,11 +860,13 @@ class UserConfiguration(Configuration):
                                                 'sms',              _OPT_PLAIN, '# How to send GPS notifications (sms|email|both):'),
 
         # Miscellaneous settings
+        'SAVEDAUDIBILITY':  ('',                '',                 _OPT_PLAIN, '# Audibility information to be used by COM_SILENTOFF (internal use only):'),
         'SILENCEDEVICE':    (_BoolPattern,      'no',               _OPT_PLAIN, '# Set phone silent before executing command (yes|no):'),
         'AUTODEVICELOCK':   (_BoolPattern,      'no',               _OPT_PLAIN, '# Lock phone before executing any command except un/lock (yes|no):'),
-        'SIMUNLOCK':        (pat('^(no|yes|locknewsim|lockalways)$').help('Must be one of "no", "yes", "locknewsim" or "lockalways"'),
-                                                'no',               _OPT_PLAIN, '# Automatic device un/locking: (no|yes|locknewsim|lockalways):'),
-        'DEBUGFLAGS':       ('',                '',                 _OPT_PLAIN, '# A csv with debugging options (for development use only)'),
+        'SIMUNLOCK':        (pat('^(no|yes|locknewsim|lockifnopin|lockifnosim|lockalways)$').help('Must be one of "no", "yes", "locknewsim", "lockifnopin", "lockifnosim" or "lockalways"'),
+                                                'no',               _OPT_PLAIN, '# Automatic device un/locking: (no|yes|locknewsim|lockifnopin|lockifnosim|lockalways):'),
+        'STOLENIFNOSIM':    (_BoolPattern,      'yes',              _OPT_PLAIN, '# Activate stolen mode if no SIM is present:'),
+        'DEBUGFLAGS':       ('',                '',                 _OPT_PLAIN, '# A csv with debugging options (for development use only):'),
     
         # Sensor detections
         'KEYBOARDDETECT':   (_BoolPattern,      'no',               _OPT_PLAIN, '# Send notification if keyboard is sliding (yes|no):'),
@@ -832,6 +874,7 @@ class UserConfiguration(Configuration):
 
         # Obsolete settings: 
         'DISABLESMS':       (None,              None,               _OPT_PLAIN, 'setting removed since version 0.8.0-1'),
+        'SIMAUTHORIZATION': (None,              None,               _OPT_PLAIN, 'setting removed')
     }
     _Secret = '6983456712936753'
 
@@ -910,9 +953,10 @@ class UserConfiguration(Configuration):
 
 class IMSIConfiguration(Configuration):
     """
-    Instances of this class represent the (one and only) IMSI code database
+    Instances of this class represent the (one and only) database of authorized SIM cards.
     """ 
-    ItemTypeName        = "IMSI code"
+    ItemTypeName        = "SIM code"
+    Thing               = "SIM database"
     ConfigPrintFormat   = "%s%s"
     ConfigParseFormat   = "^\\s*(.*?)\\s*()()$"
     TabSize             = 18
@@ -922,25 +966,43 @@ class IMSIConfiguration(Configuration):
 
     def _genConfigLine(self, ConfigVar, ConfigValue):
         return ConfigVar
+    
+    def PackCodeType(self, CodeType, ConfigValue):
+        """Assemble type and value to a string actually stored in database."""
+        if CodeType == 'IMSI':  return ConfigValue
+        else:                   return CodeType + ':' + ConfigValue
+
+    def UnpackCodeType(self, ConfigValue):
+        """Disassemble string value from database to real type and value."""
+        CodeType, sep, CodeValue = ConfigValue.partition(':')
+        if sep: return [CodeType, CodeValue]
+        else:   return ['IMSI', ConfigValue]
+
+    def updateCode(self, CodeType, ConfigName, ConfigValue, Silent = False):
+        return self.update(self.PackCodeType(CodeType, ConfigName), ConfigValue, Silent)
 
 def GetPhoneCode(CodeType):
     """
-    Get current SIM IMSI code or device hardware IMEI code.
-    @return string|None The requested code.
+    Get device hardware IMEI code (always available)
+    or current SIM ICCID code (available if SIM present, et all)
+    or SIM IMSI code (available if correct SIM PIN was entered).
+    @return string|None The requested code. None if code is not available.
     """
     if CodeType == 'IMEI':
         Command = 'dbus-send --system --type=method_call --print-reply --dest=com.nokia.phone.SIM /com/nokia/phone/SIM/security Phone.Sim.Security.get_imei'
+    elif CodeType == 'ICCID':
+        Command = 'dbus-send --system --type=method_call --print-reply --dest=com.nokia.phone.SSC /com/nokia/phone/SSC com.nokia.phone.SSC.get_iccid'
     elif CodeType == 'IMSI':
         Command = 'dbus-send --system --type=method_call --print-reply --dest=com.nokia.phone.SIM /com/nokia/phone/SIM Phone.Sim.get_imsi'
     else:
-        LOGGER.error('unknown code type "%s"' % CodeType)
+        LOGGER.error('Unknown code type "%s"' % CodeType)
         return None
 
     try:
-        E = re.compile('string "(\S+)"')
+        E = re.compile('string "(\S+)"')    # need non-empty string!
         return E.findall(os.popen(Command).read())[0]
     except:
-        LOGGER.error('could not get %s code.' % CodeType)
+        LOGGER.warning('No %s code available.' % CodeType)
         return None
 
 ##############################################################
@@ -1271,7 +1333,7 @@ netstat -tean
         elif Force:
             LOGGER.warning('Forced to create new %s.' % Thing)
         else:
-            LOGGER.info('The %s already exists. Not initialized, left unchanged.' % Thing)
+            LOGGER.info('The %s already exists. Not changed.' % Thing)
             return True
 
         try:
@@ -1336,9 +1398,12 @@ pre-start script
     done
 end script
 
-script                                                           
-    exec "%(DaemonFile)s" > "%(LogFile)s" 2>&1
-end script 
+script
+    test -e    "%(LogFile)s" || exec "%(DaemonFile)s"
+    >          "%(LogFile)s"
+    chmod 0600 "%(LogFile)s"
+    exec "%(DaemonFile)s" >> "%(LogFile)s" 2>&1
+end script
 
 respawn
 """ % {'NAME': NAME, 'VERSION': VERSION, 'InstPath': InstPath,
@@ -1346,6 +1411,10 @@ respawn
        'LogFile'    : os.path.join(InstPath, NAME+'_boot.log')
        }
 
+    # Above script does the following: Log the daemon output in smscon_boot.log only
+    # if a smscon_boot.log did exist already. Since installation of smscon 0.10.4 such
+    # a file does normally no longer exist. It may be created for debugging purposes.
+
     Filename = os.path.join(BootPath, BootFile)
     if IsFileReadable(Filename):
         LOGGER.info('Boot file "%s" already exists. Not changed.' % Filename)
@@ -1364,6 +1433,37 @@ respawn
 
 ##############################################################
 
+class OurRotatingFileHandler(logging.handlers.RotatingFileHandler):
+    """
+    A RotatingFileHandler that cares for umask when we need it.
+    """
+    def doRollover(self):
+        """
+        Do a rollover, as described in __init__() of /usr/lib/python2.5/logging/handlers.py
+        but use self.mode to open the file and use umask to hide log from world.
+        """
+        self.stream.close()
+        if self.backupCount > 0:
+            for i in range(self.backupCount - 1, 0, -1):
+                sfn = "%s.%d" % (self.baseFilename, i)
+                dfn = "%s.%d" % (self.baseFilename, i + 1)
+                if os.path.exists(sfn):
+                    #print "%s -> %s" % (sfn, dfn)
+                    if os.path.exists(dfn):
+                        os.remove(dfn)
+                    os.rename(sfn, dfn)
+            dfn = self.baseFilename + ".1"
+            if os.path.exists(dfn):
+                os.remove(dfn)
+            os.rename(self.baseFilename, dfn)
+            #print "%s -> %s" % (self.baseFilename, dfn)
+        umaskOld = os.umask(0137)
+        if self.encoding:
+            self.stream = codecs.open(self.baseFilename, self.mode, self.encoding)
+        else:
+            self.stream = open(self.baseFilename, self.mode)
+        os.umask(umaskOld)
+
 def StartLogging(instanceName, logLevel = None, withConsole = True, withEmitterFxn = False):
     """
     Start logging for specified instance at specified level (default all)
@@ -1379,7 +1479,8 @@ def StartLogging(instanceName, logLevel = None, withConsole = True, withEmitterF
 
     if os.geteuid() == 0:
         setattr(logger, 'LogFilename', os.path.join(LogPath, LogFile))
-        fh = logging.FileHandler(logger.LogFilename)
+#        fh = logging.FileHandler(logger.LogFilename, 'a')
+        fh = OurRotatingFileHandler(logger.LogFilename, 'a', 96*1024, 5)
 #        fh.setLevel(logLevel)            -- NO! dynamically inherit logLevel from parent
         fh.setFormatter(logging.Formatter('%(asctime)s[%(name)s] %(levelname)s: %(message)s', datefmt = TimeFormat))
         logger.addHandler(fh)
